<?php

class adb {

    private $server = "localhost";
    private $user = "root";
    private $pass = "";
    private $db = "tienda";
    private $log = "errores.log";
    private $conn;
    private $pagina = 5;
    private $tablas = ['clientes' => ['idclientes', 'mail', 'password', 'direccion', 'cpostal', 'ciudad'],
        'productos' => ['idproductos', 'nombre', 'precio', 'descripcion'],
        'pedidos' => ['idpedidos', 'idcliente', 'fecha', 'importe', 'iva']
    ];
    public $numFilas;
    public $filas;

    function __construct() {
        $this->conectar();
    }

    function conectar() {
        try {
            $this->conn = new PDO("mysql:host=$this->server;dbname=$this->db;charset=utf8", $this->user, $this->pass);
            $this->conn->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
            $this->conn->query("SET NAMES 'utf8';");
        } catch (PDOException $ex) {
            $this->tratarError("conectar", $ex->getMessage());
            die($ex->getMessage());
        }
    }

    function crear_tabla($tabla, $pag) {
        $this->cabecera_tabla($tabla);
        $this->filas_tabla($tabla, $pag);
        $this->pie_tabla($tabla, $pag);
    }

    function cabecera_tabla($tabla) {
        ?>
        <table class="table table-striped">
            <thead>
                <tr>
                    <th>Id</th>
                    <th><?= ucfirst($this->tablas[$tabla][1]) ?></th>
                    <th><?= ucfirst($this->tablas[$tabla][2]) ?></th>
                    <th>Acciones</th>
                </tr>
            </thead>
            <tbody>
                <?php
            }

            function filas_tabla($tabla, $pag) {
                $inicio = ($pag - 1) * $this->pagina;
                $sql = "SELECT * from $tabla limit $inicio,$this->pagina";
                $filas = $this->getAllRows($sql);
                foreach ($filas as $fila) {
                    $this->fila_tabla($fila);
                }
            }

            function fila_tabla($fila) {
                ?>
                <tr><td><?= $fila[0] ?></td><td><?= $fila[1] ?></td><td><?= $fila[2] ?></td>
                    <td><a href="mantenimiento.php?action=editar&id=<?= $fila[0] ?>" class="btn btn-default btn-sm">
                            <span class="glyphicon glyphicon-pencil"></span> Editar 
                        </a> <a  href="mantenimiento.php?action=borrar&id=<?= $fila[0] ?>" class="btn btn-default btn-sm">
                            <span class="glyphicon glyphicon-trash"></span> Borrar 
                        </a></td></tr>
                <?php
            }

            function pie_tabla($tabla, $pag) {
                $total = $this->getCount($tabla);
                $paginas = ($total - 1) / $this->pagina;
                ?>
                <tr><td colspan="4">
                        <ul class="pagination" id="mipag">
                            <?php
                            for ($i = 1; $i <= $paginas + 1; $i++) {
                                ?>
                                <li <?= ($pag == $i) ? 'class="active"' : '' ?>>
                                    <a  class="btn btn-success" href="?pag=<?= $i ?>"><?= $i ?></a>
                                </li>
                                <?php
                            }
                            ?>

                        </ul></td></tr>
                <?php
            }

            function selector_tablas() {
                ?><label for="tablas">Seleccione tabla:</label>
            <select onchange="location.href = '?tabla=' + this.value" name="tablas">
                <option>Seleccione una tabla</option>
                <?php
                foreach ($this->tablas as $clave => $valor) {
                    echo '<option>' . $clave . '</option>';
                }
                ?>
            </select>
            <?php
        }

        function checkAction($tabla) {
            $action = filter_input(INPUT_POST, 'action', FILTER_SANITIZE_STRING);
            if (empty($action))
            { return;}
            $fila = $this->getFilaPost($tabla);
            if ($action == 'editar') {
                $this->editar_fila($tabla, $fila);
                
            }
            if ($action == 'nuevo') {
                $this->nueva_fila($tabla, $fila);
            }
            if ($action == 'borrar') {
                $this->elimina_fila($tabla, $fila);
            }
        }

        function editar_fila($tabla, $fila) {
            $sql = "update $tabla set ";
            $sql.=$this->campos_update($tabla);
            $sql.=$this->campos_where($tabla);

            if ($this->ejecuta($sql, $fila)) {
                $this->html_exito("Fila editada");
                $this->subir_imagen($fila[$this->tablas[$tabla][0]]);
            } else {
                $this->html_error("No se ha podido editar la fila");
            }
        }

        function elimina_fila($tabla, $fila) {
            $sql = "delete from $tabla ";
            $sql.=$this->campos_where($tabla);
            if ($this->ejecuta($sql, array($this->tablas[$tabla][0] => $fila[$this->tablas[$tabla][0]]))) {
                $this->html_exito("Fila eliminada");
            } else {
                $this->html_error("No se ha podido eliminar la fila");
            }
        }

        function nueva_fila($tabla, $fila) {
            $sql = "insert into $tabla ";
            $sql.=$this->campos_insert($tabla);

            if ($this->ejecuta($sql, $fila)) {
                $this->html_exito("Fila Añadida");
                $this->subir_imagen($this->conn->lastInsertId());
            } else {
                $this->html_error("No se ha podido editar la fila");
            }
        }

        function getFilaPost($tabla) {
            foreach ($this->tablas[$tabla] as $campo) {
                $fila[$campo] = filter_input(INPUT_POST, $campo);
            }
            return $fila;
        }

        function campos_update($tabla) {
            $cad = "";
            foreach ($this->tablas[$tabla] as $campo) {
                $cad.=$campo . "=:" . $campo . ",";
            }
            return substr($cad, 0, -1);
        }

        function campos_insert($tabla) {
            $cad1 = "";
            $cad2 = "";
            foreach ($this->tablas[$tabla] as $campo) {
                $cad1.=$campo . ",";
                $cad2.=":" . $campo . ",";
            }

            return "(" . substr($cad1, 0, -1) . ") values (" . substr($cad2, 0, -1) . ")";
        }

        function campos_where($tabla) {
            $id = $this->tablas[$tabla][0];
            $cad = " where " . $id . "=:" . $id;
            return $cad;
        }

        function ejecuta($sql, $fila) {
            try {

                $st = $this->conn->prepare($sql);

                $st->execute($this->preparaArray($fila));
                return $st->rowCount();
            } catch (PDOException $e) {
                return false;
            }
        }

        function preparaArray($fila) {
            foreach ($fila as $campo => $valor) {
                $tabla[':' . $campo] = $valor;
            }
            return $tabla;
        }

        function getAllRows($sql) {
            try {

                $stmt = $this->conn->prepare($sql);
                $stmt->execute();
                $filas = $stmt->fetchAll();

                return $filas;
            } catch (Exception $ex) {
                return array();
            }
        }

        function getElement($tabla, $id) {
            $sql = "SELECT * from $tabla where " . $this->tablas[$tabla][0] . "=" . $id;
            $filas = $this->getAllRows($sql);
            return $filas[0];
        }

        function getCount($tabla) {
            $sql = "SELECT count(*) as total from $tabla";
            $stmt = $this->conn->prepare($sql);
            $stmt->execute();
            $fila = $stmt->fetch();
            return $fila['total'];
        }

        function html_error($texto) {
            ?> <div class="alert alert-danger">
                <strong>Error: </strong> <?= $texto ?>.
            </div>
            <?php
        }

        function html_exito($texto) {
            ?> <div class="alert alert-success">
                <strong>Correcto: </strong> <?= $texto ?>.
            </div>
            <?php
        }

        function formulario($tabla, $fila, $action) {
            $form = file_get_contents($tabla . ".tpl");
            if ($action == 'nuevo') {
                $form = preg_replace("(\#.*\#)", "", $form);
            } else {
                $form = $this->rellena_tpl($form, $fila);
            }
            ?>
            <form role="form" action="index.php" enctype="multipart/form-data" method="post">
                <input type="hidden" name="action" value="<?= $action ?>">
                <?= $form ?>
            </form>
            <?php
        }

        function rellena_tpl($form, $fila) {
            foreach ($fila as $campo => $valor) {
                $form = str_replace("#$campo#", $valor, $form);
            }
            return $form;
        }

        function subir_imagen($id) {
            if (!empty($_FILES['foto']) && is_uploaded_file($_FILES['foto']['tmp_name'])) {
                $dir_subida = 'imgprod/';
                $fichero_subido = $dir_subida . $id . ".jpg";

                if (!@move_uploaded_file($_FILES['foto']['tmp_name'], $fichero_subido)) {
                    return false;
                }
            }
            return true;
        }

    }
    